<?php

class PayshopCreatePaymentOrder
{
    /**
     * @var PayshopClient
     */
    private $payshopSDK;

    /**
     * Class constructor
     */
    public function __construct()
    {
        $this->payshopSDK = PayshopClientFactory::getInstance();
    }

    /**
     * Create a payment order in the gateway
     *
     * @param WC_Order $order
     * @return string
     * @throws Exception
     */
    public function execute($order)
    {
        $this->validateCredentials();
        $this->validateCurrency($order);

        $servicePaymentId = PayshopCommons::getPaymentServiceUUID(
            $order->get_payment_method()
        );

        $response = $this->payshopSDK->createPaymentOrder(array_merge([
            'amount' => $order->get_total() * 100,
            'currency' => 'EUR',
            'operative' => $this->getOperativeMode($order),
            'service' => $servicePaymentId,
    
            "description" => $this->getDescription($order),

            'url_ok' => $order->get_checkout_order_received_url(),
            'url_ko' =>  $this->processFailedRedirectURL($order),
            "url_post" => $this->webHookProcessEventURL(),
        ], $this->getPaymentData($order)));
    
        PayshopCommons::checkResponse($response);

        $this->validateGatewayCurrency($response);

        return $response['response']['order'];
    }

    /**
     * Validate credentials
     *
     * @return void
     * @throws Exception
     */
    private function validateCredentials()
    {
        if (!$this->payshopSDK) {
            $message = __('Check your credentials on Payshop setting page', 'payshop');
            PayshopLog::generate($message, PayshopLog::LOG_SEVERITY_ERROR);
            throw new Exception($message);
        }
    }

    /**
     * Validate currency
     *
     * @param WC_Order $order
     * @return void
     * @throws Exception
     */
    private function validateCurrency($order)
    {
        $currecy = $order->get_currency();

        if ($currecy != 'EUR') {
            $message = __('Product currency must be EUR', 'payshop');
            PayshopLog::generate($message, PayshopLog::LOG_SEVERITY_ERROR);
            throw new Exception($message);
        }
    }

    /**
     * Get description
     * 
     * @param WC_Order $order
     * @return string
     */
    private function getDescription($order)
    {
        $shopName = get_bloginfo('name');
        
        return $shopName . __(' - order #', 'payshop') . $order->get_id();
    }

    /**
     * Get payment data
     * 
     * @param WC_Order $order
     * @return array
     */
    private function getPaymentData($order)
    {
        if ($order->get_payment_method() === 'payshop_card') {
            return [
                'secure' => true,
                'save_card' => false,
            ];
        }

        $options['secure'] = false;

        if ($order->get_payment_method() === 'payshop_mbway') {
            $options['extra_data'] = [
                "profile" => [
                    "first_name" => $order->get_billing_first_name(),
                    "last_name" => $order->get_billing_last_name(),
                    "phone" => [
                        "number" => trim($_POST['phone']),
                        "prefix" => trim(str_replace('+', '', $_POST['prefix']))
                    ]
                ]
            ];
        }

        return $options;
    }

    /**
     * Return the correct operative mode
     *
     * @param WC_Order $order
     * @return string
     */
    private function getOperativeMode($order)
    {
        $isCard = $order->get_payment_method() === 'payshop_card';
        $isDeferred = $order->get_meta('_payshop_payment_in_two_steps', true) == 'yes';

        if ($isCard && $isDeferred) {
            return 'DEFERRED';
        }

        return 'AUTHORIZATION';
    }

    /**
     * Get webhook process events url
     *
     * @return string
     */
    private function webHookProcessEventURL()
    {
        return get_site_url() . '/index.php?wc-api=payshop_process_webhook';
        //return 'https://y0twmeylub.sharedwithexpose.com' . '?wc-api=payshop_process_webhook';
    }

    /**
     * Get KO page url to redirect the user
     * @param WC_Order $order
     * @return string
     */
    private function processFailedRedirectURL($order)
    {
        return get_site_url() . '?wc-api=payshop_process_failed_redirect&order_id='. $order->get_id();
    }

    /**
     * Validate currency sent by the gateway
     * 
     * @param array $response
     * @return void
     * 
     * @throws Exception
     */
    private function validateGatewayCurrency($response)
    {
        if ($response['response']['order']['currency'] != '978') {
            $message = __('Product currency must be EUR', 'payshop');
            PayshopLog::generate($message, PayshopLog::LOG_SEVERITY_ERROR);
            throw new Exception($message);
        }
    }
}