<?php

class PayshopUpdateAlert extends PayshopAbstractAlert
{

    /**
     * @var string
     */
    const ONLINE_VERSION_URL = 'https://apps.coolsis.pt/payshop/woocommerce/version.json';

    /**
     * Return message to display if the module is not updated
     *
     * @return string|void
     */
    public function execute()
    {
        if (!extension_loaded('curl')) {
            return;
        }

        if ($this->alreadyShownInTheLast24hours()) {
            return;
        }
        
        $onlineVersionInformations = $this->getOnlineVersion();
        
        if ($this->isUpdated($onlineVersionInformations['version'])) {
            return;
        }

        $this->show('error', $this->message($onlineVersionInformations), [
            'update' => true
        ]);
    }

    /**
     * Check if the module is updated
     *
     * @param string $onlineVersion
     * @return boolean
     */
    private function isUpdated($onlineVersion)
    {
        return version_compare(
            PAYSHOP_PLUGIN_VERSION,
            $onlineVersion,
            '=='
        );
    }

    /**
     * Get version from online URL
     *
     * @return array
     */
    private function getOnlineVersion()
    {
        $ch = curl_init();
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
        curl_setopt($ch, CURLOPT_URL, self::ONLINE_VERSION_URL);
        $data = curl_exec($ch);
        curl_close($ch);
        $current_version = $data;

        $versionInformation = json_decode($current_version, true);

        return $versionInformation;
    }

    /**
     * Check if the message was already shown in the last 24 hours
     *
     * @return bool
     */
    private function alreadyShownInTheLast24hours()
    {
        $lastclosedAlertDate = get_option('PAYSHOP_LAST_CLOSED_ALERT_UPDATE_DATE');

        if ($lastclosedAlertDate == null) {
            return false;
        }

        $yesterday = date('Y-m-d H:i', strtotime('-1 days'));

        return $lastclosedAlertDate > $yesterday;
    }

    /**
     * Return message to display if the module is not updated
     *
     * @param array $onlineVersionInformations
     * @return string
     */
    private function message($onlineVersionInformations)
    {
        $alert = __('A new version of the Payshop module is available', 'payshop');
        $download = __('You can download the latest version from here:', 'payshop');
        
        $link = sprintf(
            '<a href="%s" target="_blank">' . __('download', 'payshop') . '</a>',
            $onlineVersionInformations['url']
        );

        return vsprintf(
            '%s %s. %s %s',
            [
                $alert,
                $onlineVersionInformations['version'],
                $download,
                $link
            ]
            );
    }
}