<?php

class PayshopCard extends WC_Payment_Gateway 
{
    /**
     * @var PayshopCreatePaymentOrder
     */
    private $payshopCreatePaymentOrder;

    /**
     * @var PayshopUpdateOrder
     */
    private $payshopUpdateOrder;

    /**
     * Define the payment gateway properties
     */
    public function __construct()
    {
        $this->id = 'payshop_card';
        $this->icon = PAYSHOP_PLUGIN_URL . 'templates/img/visa_mc.png'; 
        $this->has_fields = true; 
        $this->method_title = __('Payshop Card', 'payshop');
        $this->method_description = __('Debit and credit card on Payshop Online Payments', 'payshop'); 

        $this->supports = [
            'products'
        ];

        $this->init_form_fields();
        $this->init_settings();
        $this->title = $this->get_option('payshop_card_title');
        $this->description = $this->get_option('payshop_card_description');
        $this->enabled = $this->get_option('payshop_card_enabled');
     
        add_action('woocommerce_update_options_payment_gateways_' . $this->id, [$this, 'process_admin_options']);

        $this->payshopCreatePaymentOrder = new PayshopCreatePaymentOrder();
        $this->payshopUpdateOrder = new PayshopUpdateOrder();
    }

    /**
     * Define configuration fields for the payment gateway
     * 
     * @return void
     */
    public function init_form_fields()
    {
        $this->form_fields = array(
            'payshop_card_enabled' => array(
                'title'       => __('Enable/Disable', 'payshop'),
                'label'       => __('Enable Card Payment', 'payshop'),
                'type'        => 'checkbox',
                'description' => '',
                'default'     => 'no'
            ),
            'payshop_payment_in_two_steps' => array(
                'title'       => __('Payment in two steps', 'payshop'),
                'label'       => __('Enable Payment in two steps, first capture the amount and then merchant confirms the value', 'payshop'),
                'type'        => 'checkbox',
                'description' => '',
                'default'     => 'no'
            ),
            'payshop_card_title' => array(
                'title'       => __('Title'),
                'type'        => 'text',
                'description' => __('This controls the title which the user sees during checkout.', 'payshop'),
                'default'     => __('Credit or debit card', 'payshop'),
            ),
            'payshop_card_description' => array(
                'title'       => __('Description'),
                'type'        => 'textarea',
                'description' => __('This controls the description which the user sees during checkout.', 'payshop'),
                'default'     => __('Pay with your card via with Payshop Online Payments.', 'payshop'),
            ),
        );
    }

    /**
     * Display the payment fields on the checkout page
     * 
     * @return void
     */
    public function payment_fields()
    {
        if ($this->description) {
            if (get_option('payshop_environment') == 'sandbox') {
                $this->description .= '<br>' . __('TEST MODE ENABLED.', 'payshop');
            }

            echo wpautop(wp_kses_post($this->description));
        }

        do_action('woocommerce_payshop_card_form_start', $this->id);
        echo __('After click on confirmation button. You will see form to insert your card informations on the next page', 'payshop');
        do_action('woocommerce_payshop_card_form_end', $this->id);
    }

    /**
     * Process the payment
     * 
     * @param int $orderId
     * @return array
     */
    public function process_payment($orderId)
    {
        try {
            $order = wc_get_order($orderId);
            
            $order->update_meta_data('_payshop_payment_in_two_steps', $this->get_option('payshop_payment_in_two_steps'), true);
            $order->save();
            
            $paymentOrder = $this->payshopCreatePaymentOrder->execute($order);

            $this->payshopUpdateOrder->execute(
                    $orderId,
                    "on-hold",
                    $paymentOrder['uuid']
            );
            
            return [
                'result' => 'success',
                'redirect' => PayshopClientFactory::getInstance()->getRedirectUrl($paymentOrder['token'])
            ];
        } catch (\Throwable $th) {
            wc_add_notice($th->getMessage(), 'error');

            return [
                'result'       => 'failure',
                'errorMessage' => $th->getMessage(),
            ];
        }
    }
}
