<?php

class PayshopMBWay extends WC_Payment_Gateway 
{
    /**
     * @var PayshopCreatePaymentOrder
     */
    private $payshopCreatePaymentOrder;

    /**
     * @var PayshopUpdateOrder
     */
    private $payshopUpdateOrder;

    /**
     * Define the payment gateway properties
     */
    public function __construct()
    {
        $this->id = 'payshop_mbway';
        $this->icon = PAYSHOP_PLUGIN_URL . 'templates/img/mbway.png'; 
        $this->has_fields = true; 
        $this->method_title = __('MBWay', 'payshop');
        $this->method_description = __('Pay with MBWay', 'payshop'); 

        $this->supports = [
            'products'
        ];

        $this->init_form_fields();
        $this->init_settings();
        $this->title = $this->get_option('payshop_mbway_title');
        $this->description = $this->get_option('payshop_mbway_description');
        $this->enabled = $this->get_option('payshop_mbway_enabled');
     
        add_action('woocommerce_update_options_payment_gateways_' . $this->id, [$this, 'process_admin_options']);
        add_action('wp_enqueue_scripts', [$this, 'payment_scripts']);

        $this->payshopCreatePaymentOrder = new PayshopCreatePaymentOrder();
        $this->payshopUpdateOrder = new PayshopUpdateOrder();
    }

    /**
     * Define configuration fields for the payment gateway
     * 
     * @return void
     */
    public function init_form_fields()
    {
        $this->form_fields = array(
            'payshop_mbway_enabled' => array(
                'title'       => __('Enable/Disable', 'payshop'),
                'label'       => __('Enable MBWay Payment', 'payshop'),
                'type'        => 'checkbox',
                'description' => '',
                'default'     => 'no'
            ),
            'payshop_mbway_title' => array(
                'title'       => __('Title'),
                'type'        => 'text',
                'description' => __('This controls the title which the user sees during checkout.', 'payshop'),
                'default'     => __('MBWay', 'payshop'),
            ),
            'payshop_mbway_description' => array(
                'title'       => __('Description'),
                'type'        => 'textarea',
                'description' => __('This controls the description which the user sees during checkout.', 'payshop'),
                'default'     => __('Pay with MBWay', 'payshop'),
            ),
        );
    }

    /**
     * Enqueue the scripts and styles
     * 
     * @return void
     */
    public function payment_scripts() 
    {
        if (!is_cart() && !is_checkout() && !isset($_GET['pay_for_order'])) {
            return;
        }

        if ('no' === $this->enabled) {
            return;
        }

        wp_register_script('payshop_mbway', PAYSHOP_PLUGIN_URL . 'templates/js/mbway.js', ['jquery']);
        wp_enqueue_script('payshop_mbway');
    }
    
    /**
     * Display the payment fields on the checkout page
     * 
     * @return void
     */
    public function payment_fields()
    {
        if ($this->description) {
            if (get_option('payshop_environment') == 'sandbox') {
                $this->description .= '<br>' . __('TEST MODE ENABLED.', 'payshop');
            }

            echo wpautop(wp_kses_post($this->description));
        }

        do_action('woocommerce_mbway_form_start', $this->id);
        require_once PAYSHOP_PLUGIN_DIR . '/templates/views/front/mbway.php';
        do_action('woocommerce_mbway_form_end', $this->id);
    }

    /**
     * Process the payment
     * 
     * @param int $orderId
     * @return array
     */
    public function process_payment($orderId)
    {
        try {
            $this->phoneValidation();

            $order = wc_get_order($orderId);
            $paymentOrder = $this->payshopCreatePaymentOrder->execute($order);

            $this->payshopUpdateOrder->execute(
                    $orderId,
                    "on-hold",
                    $paymentOrder['uuid']
            );

            wc()->session->set( 'order_id', $orderId );
            
            return [
                'result' => 'success',
                'redirect' => PayshopClientFactory::getInstance()->getRedirectUrl($paymentOrder['token']) . '?apm=MBWAY'
            ];
        } catch (\Throwable $th) {
            wc_add_notice($th->getMessage(), 'error');

            return [
                'result'       => 'failure',
                'errorMessage' => $th->getMessage(),
            ];
        }
    }

    /**
     * Validate the phone number
     * 
     * @return bool
     */
    public function phoneValidation()
    {
        $mbwayprefix = isset($_POST['prefix']) ? trim($_POST['prefix']) : '';
        $mbwayPhone = isset($_POST['phone']) ? trim($_POST['phone']) : '';

        if (strlen($mbwayprefix) < 1 || strlen($mbwayprefix) > 4 || strlen($mbwayPhone) < 9) {
            throw new Exception(__('Phone number is invalid', 'payshop'));
        }
    }
}

