<?php

class PayshopReference extends WC_Payment_Gateway 
{
    /**
     * @var PayshopCreatePaymentOrder
     */
    private $payshopCreatePaymentOrder;

    /**
     * @var PayshopUpdateOrder
     */
    private $payshopUpdateOrder;

    /**
     * Define the payment gateway properties
     */
    public function __construct()
    {
        $this->id = 'payshop_reference';
        $this->icon = PAYSHOP_PLUGIN_URL . 'templates/img/payshop-reference.png'; 
        $this->has_fields = true; 
        $this->method_title = __('Payshop Reference', 'payshop');
        $this->method_description = __('Pay with Payshop Reference', 'payshop'); 

        $this->supports = [
            'products'
        ];

        $this->init_form_fields();
        $this->init_settings();
        $this->title = $this->get_option('payshop_reference_title');
        $this->description = $this->get_option('payshop_reference_description');
        $this->enabled = $this->get_option('payshop_reference_enabled');
     
        add_action('woocommerce_update_options_payment_gateways_' . $this->id, [$this, 'process_admin_options']);

        $this->payshopCreatePaymentOrder = new PayshopCreatePaymentOrder();
        $this->payshopUpdateOrder = new PayshopUpdateOrder();
    }

    /**
     * Define configuration fields for the payment gateway
     * 
     * @return void
     */
    public function init_form_fields()
    {
        $this->form_fields = array(
            'payshop_reference_enabled' => array(
                'title'       => __('Enable/Disable', 'payshop'),
                'label'       => __('Enable Payshop Reference Payment', 'payshop'),
                'type'        => 'checkbox',
                'description' => '',
                'default'     => 'no'
            ),
            'payshop_reference_title' => array(
                'title'       => __('Title'),
                'type'        => 'text',
                'description' => __('This controls the title which the user sees during checkout.', 'payshop'),
                'default'     => __('Payshop Reference', 'payshop'),
            ),
            'payshop_reference_description' => array(
                'title'       => __('Description'),
                'type'        => 'textarea',
                'description' => __('This controls the description which the user sees during checkout.', 'payshop'),
                'default'     => __('Pay with Payshop reference.', 'payshop'),
            ),
        );
    }
    
    /**
     * Display the payment fields on the checkout page
     * 
     * @return void
     */
    public function payment_fields()
    {
        if ($this->description) {
            if (get_option('payshop_environment') == 'sandbox') {
                $this->description .= '<br>' . __('TEST MODE ENABLED.', 'payshop');
            }

            echo wpautop(wp_kses_post($this->description));
        }

        do_action('woocommerce_payshop_reference_form_start', $this->id);
        echo __('After click on confirmation button. You will see reference on the next page', 'payshop');
        do_action('woocommerce_payshop_reference_form_end', $this->id);
    }

    /**
     * Process the payment
     * 
     * @param int $orderId
     * @return array
     */
    public function process_payment($orderId)
    {
        try {
            $order = wc_get_order($orderId);

            $paymentOrder = $this->payshopCreatePaymentOrder->execute($order);

            $this->payshopUpdateOrder->execute(
                    $orderId,
                    "on-hold",
                    $paymentOrder['uuid']
            );

            $iframeURL = PayshopClientFactory::getInstance()->getRedirectUrl($paymentOrder['token']) . '?apm=PAYSHOP';
            wc()->session->set( 'order_iframe_url', $iframeURL );

            return [    
                'result' => 'success',
                'redirect' => $this->get_return_url($order)
            ];
        } catch (\Throwable $th) {
            wc_add_notice($th->getMessage(), 'error');

            return [
                'result'       => 'failure',
                'errorMessage' => $th->getMessage(),
            ];
        }
    }
}
